@@ -26,6 +26,13 @@ char *a_names[]={
 	"requeue"
 };
 
+char *s_names[]={
+	"unsorted",
+	"sorted by from",
+	"sorted by to",
+	"sorted by subject"
+};
+
 int page_step;
 int MSGMARK;
 
@@ -71,6 +78,40 @@ int wnd_confirm ( const char *action ) {
 	return ret;
 }
 
+int wnd_change_sort_field () {
+	int c, ret;
+
+	WINDOW *w = newwin ( 8, 50, (LINES-8)/2, (COLS-50)/2 );
+	wnd_border ( w );
+	
+	if ( pfql_getstatus(pfql_ctx)->use_colors )
+		wbkgd ( w, COLOR_PAIR(CP_DIALOG) );
+
+	nocbreak();
+	cbreak();
+	
+	mvwprintw ( w, 1, 2, "Choose sort field:" );
+	mvwprintw ( w, 3, 2, "0: unsorted" );
+	mvwprintw ( w, 4, 2, "1: from    (4 descendent)" );
+	mvwprintw ( w, 5, 2, "2: to      (5 descendent)" );
+	mvwprintw ( w, 6, 2, "3: subject (6 descendent)" );
+
+	wrefresh ( w );
+	
+	c = wgetch ( w );
+
+	if ( c>='0' && c<='6' )
+		ret = c-'0';
+	else
+		ret = -1;
+
+	delwin ( w );
+
+	halfdelay ( half_delay_time*10 );
+	return ret;
+
+}
+
 void wnd_getregexp () {
 	char *tmp;
 	
@@ -113,7 +154,7 @@ void wnd_getregexp () {
 }
 
 void version() {
-	printf ( "pfqueue - Version %s - (C) 2004, 2005, 2006 Stefano Rivoir\n",
+	printf ( "pfqueue - Version %s - (C) 2004-2007 Stefano Rivoir\n",
 			VERSION );
 }
 
@@ -138,7 +179,7 @@ void help() {
 			"  Backend version: %s",
 			pfql_version(), pfql_backend_apiversion(pfql_ctx), pfql_backend_id(pfql_ctx), pfql_backend_version(pfql_ctx) );
 
-	w = newwin ( 25, 60, (LINES-25)/2,(COLS-60)/2 );
+	w = newwin ( 25, 60, (LINES-26)/2,(COLS-60)/2 );
 	if ( pfql_getstatus(pfql_ctx)->use_colors )
 		wbkgd ( w, COLOR_PAIR(CP_DIALOG) );
 
@@ -239,10 +280,11 @@ void win_header() {
 	if ( pfql_getstatus(pfql_ctx)->use_colors )
 		wattron ( hwnd, COLOR_PAIR(CP_HEAD) );
 
-	mvwprintw ( hwnd, 0, 1, "Queue: '%s', %d message%s, %d tagged",
+	mvwprintw ( hwnd, 0, 1, "Queue: '%s', %d message%s, %d tagged, %s",
 			pfql_queue_name(pfql_ctx,pfql_getstatus(pfql_ctx)->cur_queue), msg_num, 
 			( msg_num != 1 ? "s" : "" ),
-			pfql_num_tag(pfql_ctx) );
+			pfql_num_tag(pfql_ctx),
+			s_names[pfql_getstatus(pfql_ctx)->sort_field]);
 	wclrtoeol ( hwnd );
 
 	wattroff ( hwnd, A_REVERSE );
@@ -314,7 +356,7 @@ void win_resize () {
 	bline = (char*)malloc(sizeof(char)*COLS+1);
 	memset ( bline, ' ', sizeof(char)*COLS+1 );
 	*(bline+COLS) = 0;
-	msg_list = (struct msg_list_t*)malloc(sizeof(struct msg_list_t)*(LINES-8-2));
+	msg_list = (struct msg_list_t*)malloc(sizeof(struct msg_list_t)*(LINES));
 
 	mwnd_tot = 7;
 	lwnd_tot = LINES-mwnd_tot-2;
@@ -414,19 +456,20 @@ int msg_at(int r) {
 
 // Clears the cursor in the previous position and displays the current one
 void show_cursor() {
-	char buf[BUF_SIZE];
+	char *buf;
 	int i;
-
 	struct msg_t *msg;
 	
+	buf = (char*)malloc(BUF_SIZE);
+
 	// Current row
 	i = msg_at(CURROW);
 	mvwinnstr ( lwnd, CURROW, 0, buf, COLS );
-	if ( pfql_getstatus(pfql_ctx)->use_colors ) {
+
+	if ( pfql_getstatus(pfql_ctx)->use_colors )
 		wattron ( lwnd, COLOR_PAIR(CP_MSG) );
-	} else {
+	else
 		wattron ( lwnd, A_REVERSE );
-	}
 
 	msg = pfql_msg_at(pfql_ctx,i);
 	if ( msg && msg->tagged )
@@ -451,6 +494,7 @@ void show_cursor() {
 		mvwaddstr ( lwnd, PREVROW, 0, buf );
 	}
 
+	free ( buf );
 	wattroff ( lwnd, COLOR_PAIR(CP_MSG) );
 	wattroff ( lwnd, COLOR_PAIR(CP_TAG) );
 	wattroff ( lwnd, A_BOLD );
@@ -524,7 +568,6 @@ void msg_cat_body( WINDOW *pw, WINDOW* bw, int wl, int wc, int ctrls ) {
 		while ( j<MSG_MAX_ROWS && i < wl ) {
 			mvwaddnstr ( pw, i++, 2, lines[j++], maxcol );
 			wclrtoeol ( pw );
-			//mvwaddch ( pw, i++, COLS-4-1, ACS_VLINE );
 		}
 		wrefresh ( pw );
 
@@ -613,12 +656,11 @@ struct msg_t *msg;
 
 	msg_num = pfql_num_msg(pfql_ctx);
 	
-	//if ( dig_limit )
-	//	show_start = time(NULL);
-	//while ( j <= MAXROW && ((dig_limit && time(NULL)-show_start<=dig_limit) || !dig_limit) ) {
-	
 	last_repaint = time(NULL);
 
+	if ( pfql_getstatus(pfql_ctx)->sort_field!=PFQL_SORT_UNSORTED )
+		while ( pfql_ctx->pfql_status.queue_status!=PFQL_Q_IDLE ) { usleep(5000); }
+
 	while ( j <= MAXROW ) { 
 
 		if ( i<msg_num ) {
@@ -645,7 +687,7 @@ struct msg_t *msg;
 		} else {
 			mvwaddstr ( lwnd, j, 1, " " );
 			wclrtobot ( lwnd );
-			j = MAXROW; // Force loop exit
+			j = MAXROW+1; // Force loop exit
 		}
 		j++;
 		i++;
@@ -694,10 +736,14 @@ void fe_close() {
 }
 
 void queue_change(int q) {
-	pfql_set_queue(pfql_ctx,q);
-	CURMSG = 0;
-	goto_first();
-	win_header();
+int r;
+
+	r = pfql_set_queue(pfql_ctx,q);
+	if ( r==PFQL_OK ) {
+		CURMSG = 0;
+		goto_first();
+		win_header();
+	}
 }
 
 void msg_action(const char* id, int act) {
@@ -710,7 +756,7 @@ void msg_action(const char* id, int act) {
 }
 
 void main_loop() {
-int c;
+int c, r;
 int i;
 
 	c = 0;
@@ -781,6 +827,9 @@ int i;
 		case '2':
 			queue_change ( Q_ACTIVE );
 			break;
+		case '5':
+			queue_change ( Q_CORRUPT );
+			break;
 		case 'a':
 			pfql_tag_all(pfql_ctx);
 			queue_force_repaint ( 1 );
@@ -889,8 +938,10 @@ int i;
 			msg_show_body(0);
 			break;
 		case '.':
-			show_body_fromline++;
-			msg_show_body(0);
+			if ( show_body_win ) {
+				show_body_fromline++;
+				msg_show_body(0);
+			}
 			break;
 		case 13:
 		case 10:
@@ -902,7 +953,6 @@ int i;
 			}
 			break;
 		case 's':
-		case 'S':
 			if ( msg_num ) {
 				msg_show_body(1);
 				win_resize();
@@ -910,6 +960,18 @@ int i;
 				msg_show_headers();
 			}
 			break;
+		case 'S':
+			r = wnd_change_sort_field();
+			if ( r != -1 ) {
+				if ( r<=3 ) {
+					pfql_getstatus(pfql_ctx)->sort_field = r;
+					pfql_getstatus(pfql_ctx)->sort_sense = PFQL_SORT_ASC;
+				} else {
+					pfql_getstatus(pfql_ctx)->sort_field = r-3;
+					pfql_getstatus(pfql_ctx)->sort_sense = PFQL_SORT_DESC;
+				}
+			}
+			queue_force_repaint ( 1 );
 		}
 		
 	}
@@ -926,7 +988,7 @@ int main ( int argc, char** argv ) {
 
 	pfql_ctx = 0;
 	if ( pfql_context_create (&pfql_ctx)!=PFQL_OK ) {
-		fprintf ( stderr, "pfquque: cannot pfql_create_context!\n" );
+		fprintf ( stderr, "pfqueue: cannot pfql_create_context!\n" );
 		exit (-1);
 	}
 
