@@ -12,7 +12,9 @@
 WINDOW *mainwnd;
 WINDOW *lwnd;
 WINDOW *mwnd, *mwndb;
+WINDOW *bwnd, *bwndb;
 WINDOW *hwnd;
+int    bwnd_height;
 
 char *bline;
 
@@ -51,4 +53,6 @@ int 		fe_init();
 void		fe_close();
 const char* 	fe_current_id();
 
+void		msg_show_body();
+
 #endif
