@@ -47,7 +47,7 @@ time_t last_repaint;
 int half_delay_time;
 
 void version() {
-	printf ( "pfqueue - Version %s - (C) 2004, 2005, 2006 Stefano Rivoir\n",
+	printf ( "spfqueue - Version %s - (C) 2004-2007 Stefano Rivoir\n",
 			VERSION );
 }
 
@@ -66,112 +66,11 @@ void strip_nl(char* b, int l) {
 
 void usage() {
 	version();
-	printf ( "Usage: pfqueue [-ehvn] [-b postfix1|postfix2|exim] [-q queue_num] [-m max_msg]\n"
+	printf ( "Usage: spfqueue [-ehvn] [-b postfix1|postfix2|exim] [-q queue_num] [-m max_msg]\n"
 		 "       [-s autorefresh_seconds] [-l limit_seconds] -[B backends_path]\n"
 		 "       [-p executables_path] [-c config_path]\n" );
 }
 
-void msg_cat() {
-#if 0
-	WINDOW *w;
-	char lines[MSG_MAX_ROWS][MSG_MAX_COLS];
-	int j, i;
-	int c;
-	int fline;
-	int pgstep;
-	int msglines;
-	int ptr;
-	int buflen;
-	int maxcol;
-
-	maxcol = COLS-4-2;
-	
-	memset ( lines, 0, sizeof(lines) );
-	
-	buflen = pfql_retr_body ( msg_list[CURROW].id, cat_buf, CAT_BUF_SIZE );
-	msglines = 0;
-	ptr = 0;
-
-	// Copy cat_buf into splitted lines
-	while ( ptr<buflen && msglines < MSG_MAX_ROWS ) {
-		c = 0;
-		while ( c < maxcol && *(cat_buf+ptr)!='\n' ) {
-			lines[msglines][c] = *(cat_buf+ptr);
-			ptr++;
-			c++;
-		}
-		while ( *(cat_buf+ptr)!='\n' )
-			ptr++;
-		ptr++;
-		msglines++;
-	}
-
-	w = newwin ( LINES-4, COLS-4, 2, 2 );
-	keypad ( w, TRUE );
-	wnd_border (w);
-	pgstep = LINES-4-2-2;
-	
-	wattron ( w, A_BOLD );
-	mvwaddstr ( w, 0, 2, "Details for" );
-	mvwaddstr ( w, 0, 14, pfql_msg_at(CURMSG)->path );
-	wattroff ( w, A_BOLD );
-
-	c = 0;
-	fline = 0;
-
-	while ( c!='q' && c!='Q' ) {
-		j = fline;
-		i = 2;
-		while ( j<MSG_MAX_ROWS && i < LINES-4-1 ) {
-			mvwaddnstr ( w, i, 2, lines[j++], COLS-4-5 );
-			wclrtoeol ( w );
-			mvwaddch ( w, i++, COLS-4-1, ACS_VLINE );
-		}
-		wrefresh ( w );
-
-no_fill:
-		c = wgetch (w);
-		switch (c) {
-		case 'g':
-		case KEY_HOME:
-			fline = 0;
-			break;
-		case 'G':
-		case KEY_END:
-			fline = msglines-pgstep;
-			break;
-		case KEY_PPAGE:
-			if ( fline > pgstep )
-				fline -= pgstep;
-			else
-				fline = 0;
-			break;
-		case KEY_NPAGE:
-			if ( fline < (msglines-pgstep) )
-				fline += pgstep;
-			break;
-		case KEY_UP:
-			if ( fline )
-				fline--;
-			break;
-		case KEY_DOWN:
-			if ( fline<msglines-pgstep )
-				fline++;
-			break;
-		case 'q':
-		case 'Q':
-			break;
-		default:
-			goto no_fill;
-			break;
-		}
-	}
-	delwin ( w );
-
-	halfdelay ( half_delay_time*10 );
-#endif
-}
-
 void fe_close() {
 	if ( cat_buf )
 		free ( cat_buf );
@@ -180,14 +79,32 @@ void fe_close() {
 }
 
 void msg_action( int cs, const char* id, int act) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	pfql_msg_action(pfql_ctx, id, act);
 	sprintf ( buf, "%s: OK\n", CMD_REPLY );
 	w_socket ( cs, buf );
 }
 
+void client_send_body ( int cs, const char *id ) {
+	int buflen;
+	char cat_buf2[CAT_BUF_SIZE];
+	struct msg_t *msg;
+
+	if ( !msg ) {
+		sprintf ( cat_buf2, "%s: NOMSG\n", CMD_ERROR );
+	} else {
+		buflen = pfql_retr_body ( pfql_ctx, id, cat_buf, CAT_BUF_SIZE );
+
+		if ( buflen )
+			sprintf ( cat_buf2, "%s: %.6d,%s\n", CMD_REPLY, buflen, cat_buf );
+		else
+			sprintf ( cat_buf2, "%s: ERR\n", CMD_ERROR );
+	}
+	w_socket ( cs, cat_buf2 );
+}
+
 void client_send_from ( int cs, const char *id ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	struct msg_t *msg;
 
 	msg = pfql_msg(pfql_ctx,id);
@@ -203,8 +120,42 @@ void client_send_from ( int cs, const char *id ) {
 	w_socket ( cs, buf );
 }
 
+void client_send_status ( int cs, const char *id ) {
+	char buf[BUF_SIZE];
+	struct msg_t *msg;
+
+	msg = pfql_msg(pfql_ctx,id);
+
+	if ( !msg ) {
+		sprintf ( buf, "%s: NOMSG\n", CMD_ERROR );
+		w_socket ( cs, buf );
+		return;
+	}
+
+	pfql_retr_status(pfql_ctx,id);
+	sprintf ( buf, "%s: %s\n", CMD_REPLY, msg->stat );
+	w_socket ( cs, buf );
+}
+
+void client_send_path ( int cs, const char *id ) {
+	char buf[BUF_SIZE];
+	struct msg_t *msg;
+
+	msg = pfql_msg(pfql_ctx,id);
+
+	if ( !msg ) {
+		sprintf ( buf, "%s: NOMSG\n", CMD_ERROR );
+		w_socket ( cs, buf );
+		return;
+	}
+
+	pfql_retr_headers(pfql_ctx,id);
+	sprintf ( buf, "%s: %s\n", CMD_REPLY, msg->path );
+	w_socket ( cs, buf );
+}
+
 void client_send_to ( int cs, char *id ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	struct msg_t *msg;
 
 	msg = pfql_msg(pfql_ctx,id);
@@ -221,7 +172,7 @@ void client_send_to ( int cs, char *id ) {
 }
 
 void client_send_subj ( int cs, char *id ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	struct msg_t *msg;
 
 	msg = pfql_msg(pfql_ctx,id);
@@ -238,26 +189,26 @@ void client_send_subj ( int cs, char *id ) {
 }
 
 void client_send_nummsg ( int cs ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	sprintf ( buf, "%s: %.05d\n", CMD_REPLY, pfql_num_msg(pfql_ctx) );
 	w_socket(cs, buf);
 }
 
 void client_send_numq ( int cs ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	sprintf ( buf, "%s: %.05d\n", CMD_REPLY, pfql_num_queues(pfql_ctx) );
 	w_socket(cs, buf);
 }
 
 void client_send_queue_name ( int cs, int q ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	sprintf ( buf, "%s: %s\n", CMD_REPLY, pfql_queue_name(pfql_ctx,q) );
 	w_socket(cs, buf);
 }
 
 void client_send_id ( int cs, int n ) {
 	struct msg_t *msg;
-	char buf[255];
+	char buf[BUF_SIZE];
 	msg = pfql_msg_at(pfql_ctx,n);
 	
 	if ( (!msg) || !strlen(msg->id) )
@@ -267,8 +218,15 @@ void client_send_id ( int cs, int n ) {
 	w_socket ( cs, buf );
 }
 
+void client_send_lastchanged ( int cs ) {
+	char buf[BUF_SIZE];
+	sprintf ( buf, "%s: %d\n", CMD_REPLY, pfql_queue_last_changed(pfql_ctx) );
+	w_socket ( cs, buf );
+
+}
+
 void client_set_queue ( int cs, int q ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	pfql_set_queue(pfql_ctx,q);
 	sprintf ( buf, "%s: OK\n", CMD_REPLY );
 	w_socket ( cs, buf );
@@ -301,7 +259,7 @@ int iscmd ( const char* b, const char* c) {
 }
 
 void client_process ( int cs ) {
-	char buf[255];
+	char buf[BUF_SIZE];
 	int ex, done, bl;
 	int i;
 	
@@ -352,6 +310,16 @@ void client_process ( int cs ) {
 			done=1;
 		}
 
+		if (iscmd( buf, CMD_STATUS )) {
+			client_send_status( cs, buf+strlen(CMD_STATUS)+1 );
+			done=1;
+		}
+
+		if (iscmd( buf, CMD_PATH )) {
+			client_send_path( cs, buf+strlen(CMD_PATH)+1 );
+			done=1;
+		}
+
 		if (iscmd( buf, CMD_FROM )) {
 			client_send_from( cs, buf+strlen(CMD_FROM)+1 );
 			done=1;
@@ -367,6 +335,10 @@ void client_process ( int cs ) {
 			done=1;
 		}
 
+		if (iscmd( buf, CMD_BODY )) {
+			client_send_body( cs, buf+strlen(CMD_BODY)+1 );
+		}
+
 		if (iscmd( buf, CMD_QUIT )) {
 			ex = 1;
 			done=1;
@@ -413,158 +385,6 @@ struct sockaddr_in clia;
 			client_process ( clis );
 			shutdown ( clis, SHUT_RDWR );
 	}
-
-#if 0
-		if ( queue_needs_repaint() )
-			queue_show();
-		else
-			show_cursor();
-
-		switch ( c ) {
-		case 'G':
-		case KEY_END:
-			queue_force_repaint ( goto_last() );
-			break;
-		case 'g':
-		case KEY_HOME:
-			queue_force_repaint ( goto_first() );
-			break;
-		case KEY_UP:
-			queue_force_repaint ( goto_prev() );
-			break;
-		case KEY_DOWN:
-			queue_force_repaint ( goto_next() );
-			break;
-		case KEY_PPAGE:
-			queue_force_repaint ( goto_prevpage() );
-			break;
-		case KEY_NPAGE:
-			queue_force_repaint ( goto_nextpage() );
-			break;
-		case KEY_RESIZE:
-			win_resize();
-			break;
-		case 'e':
-			pfql_toggle_envelope();
-			queue_force_repaint ( 1 );
-			break;
-		case 'd':
-		case KEY_DC:
-			msg_action( PFQL_CURRENT, MSG_DELETE );
-			break;
-		case 'h':
-			msg_action( PFQL_CURRENT, MSG_HOLD );
-			break;
-		case 'l':
-			msg_action ( PFQL_CURRENT, MSG_RELEASE );
-			break;
-		case 'r':
-			msg_action ( PFQL_CURRENT, MSG_REQUEUE );
-			break;
-		case '1':
-			queue_change ( Q_DEFERRED );
-			break;
-		case '4':
-			queue_change ( Q_HOLD );
-			break;
-		case '3':
-			queue_change ( Q_INCOMING );
-			break;
-		case '2':
-			queue_change ( Q_ACTIVE );
-			break;
-		case 'a':
-			pfql_tag_all();
-			queue_force_repaint ( 1 );
-			win_header();
-			break;
-		case '.':
-			pfql_msg_tag(PFQL_CURRENT);
-			break;
-		case 32:
-		case 't':
-			if ( !msg_num )
-				break;
-			if ( MSGMARK==-1 ) {
-				pfql_msg_toggletag(PFQL_CURRENT);
-			} else {
-				if ( CURMSG>=MSGMARK ) {
-					for ( i=MSGMARK; i<=CURMSG; i++ )
-						pfql_msg_tag(pfql_msg_at(i)->id);
-				} else {
-					for ( i=CURMSG; i<=MSGMARK; i++ ) 
-						pfql_msg_tag(pfql_msg_at(i)->id);
-				}
-			}
-			MSGMARK = -1;
-			goto_next();
-			queue_force_repaint ( 1 );
-			show_cursor();
-			win_header();
-			break;
-		case 'T':
-			pfql_msg_searchandtag(regexps);
-			queue_force_repaint ( 1 );
-			win_header();
-			break;
-		case 'u':
-			pfql_tag_none();
-			win_header();
-			queue_force_repaint ( 1 );
-			break;
-		case ':':
-			pfql_getstatus()->auto_wrk_tagged = !pfql_getstatus()->auto_wrk_tagged;
-			win_header();
-			break;
-		case ';':
-			pfql_getstatus()->wrk_tagged = !pfql_getstatus()->wrk_tagged;
-			win_header();
-			break;
-		case '/':
-			i = pfql_msg_search(regexps);
-			goto_msg ( i );
-			queue_force_repaint ( 1 );
-			break;
-		case 'n':
-			i = pfql_msg_searchnext(regexps);
-			goto_msg ( i );
-			break;
-		case 'p':
-			pfql_msg_searchprev(regexps);
-			break;
-		case '?':
-			help();
-			queue_force_repaint ( 1 );
-			break;
-		case 'c':
-			pfql_getstatus()->ask_confirm = !pfql_getstatus()->ask_confirm;
-			win_header();
-			break;
-		case '-':
-			pfql_getstatus()->do_scan = !pfql_getstatus()->do_scan;
-			win_header();
-			break;
-		case '+':
-			pfql_getstatus()->use_colors = !pfql_getstatus()->use_colors;
-			queue_force_repaint ( 1 );
-			win_resize();
-			win_header();
-			break;
-		case 13:
-		case 's':
-		case 'S':
-			if ( msg_num ) {
-				msg_cat();
-				win_resize();
-				queue_force_repaint ( 1 );
-				msg_show();
-			}
-			break;
-		}
-		
-	}
-#endif
-
 }
 
 void sig_clean_exit(int i) {
@@ -574,24 +394,31 @@ void sig_clean_exit(int i) {
 	exit(0);
 }
 
+int fe_init() {
+	cat_buf = 0;
+	cat_buf = (char*)malloc(CAT_BUF_SIZE);
+	if ( !cat_buf )
+		return 0;
+}
+
 int main ( int argc, char** argv ) {
 	int opt;
 	int mm;
 	int clil;
 
 	if ( getuid()!=0 ) {
-		fprintf ( stderr, "You need to be root to use pfqueue, sorry\n" );
+		fprintf ( stderr, "You need to be root to use spfqueue, sorry\n" );
 		exit (-3);
 	}
 
 	pfql_ctx = 0;
 	if ( pfql_context_create(&pfql_ctx)!=PFQL_OK ) {
-		fprintf ( stderr, "pfqueue: cannot pfqueue_create_context!\n" );
+		fprintf ( stderr, "spfqueue: cannot pfqueue_create_context!\n" );
 		exit (-1);
 	}
 
 	if ( pfql_init(pfql_ctx)!=PFQL_OK ) {
-		fprintf ( stderr, "pfqueue: cannot pfql_init!\n" );
+		fprintf ( stderr, "spfqueue: cannot pfql_init!\n" );
 		exit (-1);
 	}
 	
@@ -604,18 +431,25 @@ int main ( int argc, char** argv ) {
 	/* Trap SIGINT */
 	signal ( SIGINT, sig_clean_exit );
 
+	if ( !fe_init() ) {
+		pfql_context_destroy(pfql_ctx);
+		fe_close();
+		fprintf ( stderr, "spfqueue: cannot fe_init!\n" );
+		exit (-2);
+	}
+
 	// Initialize socket
 	clil = sizeof(struct sockaddr_in);
 	svrs = socket ( AF_INET, SOCK_STREAM, 0 );
 	if ( svrs<0 ) {
-		fprintf ( stderr, "pfqueue: cannot create socket\n" );
+		fprintf ( stderr, "spfqueue: cannot create socket\n" );
 		return -1;
 	}
 	svra.sin_family = AF_INET;
 	svra.sin_addr.s_addr = INADDR_ANY;
 	svra.sin_port = htons(20000);
 	if ( bind(svrs, (struct sockaddr*)&svra, sizeof(svra)) < 0 ) {
-		fprintf ( stderr, "pfqueue: cannot bind socket\n" );
+		fprintf ( stderr, "spfqueue: cannot bind socket\n" );
 		return -1;
 	}
 	listen ( svrs, 5 );
