@@ -32,6 +32,21 @@ int dig_suspend;
 #define TH_STOPPED        2
 int thread_control;
 
+int tmp_sort_sense;
+
+// Compare callbacks for qsort
+int msg_compare_from ( const void *m1, const void *m2 ) {
+	return tmp_sort_sense * strcmp ( ((struct msg_t*)m1)->from, ((struct msg_t*)m2)->from );
+}
+
+int msg_compare_to ( const void *m1, const void *m2 ) {
+	return tmp_sort_sense * strcmp ( ((struct msg_t*)m1)->to, ((struct msg_t*)m2)->to );
+}
+
+int msg_compare_subject ( const void *m1, const void *m2 ) {
+	return tmp_sort_sense * strcmp ( ((struct msg_t*)m1)->subj, ((struct msg_t*)m2)->subj );
+}
+
 struct pfql_status_t* pfql_getstatus( struct pfql_context_t *ctx ) {
 	return &ctx->pfql_status;
 }
@@ -48,6 +63,16 @@ const char* pfql_queue_name( struct pfql_context_t *ctx, int i) {
 	return ctx->pfqbe_queue_name(i);
 }
 
+void pfql_queue_sort ( struct pfql_context_t *ctx ) {
+	tmp_sort_sense = ctx->pfql_status.sort_sense;
+	if ( ctx->pfql_status.sort_field==PFQL_SORT_FROM )
+		qsort ( ctx->queue, ctx->NUMMSG, sizeof(struct msg_t), msg_compare_from );
+	if ( ctx->pfql_status.sort_field==PFQL_SORT_TO )
+		qsort ( ctx->queue, ctx->NUMMSG, sizeof(struct msg_t), msg_compare_to );
+	if ( ctx->pfql_status.sort_field==PFQL_SORT_SUBJECT )
+		qsort ( ctx->queue, ctx->NUMMSG, sizeof(struct msg_t), msg_compare_subject );
+}
+
 time_t pfql_queue_last_changed( struct pfql_context_t *ctx ) {
 	return ctx->queue_last_changed;
 }
@@ -171,6 +196,9 @@ void* queue_fill_thread(void *arg) {
 	struct pfql_context_t *ctx = (struct pfql_context_t*)arg;
 
 	while ( thread_control == TH_RUNNABLE ) {
+		
+		ctx->pfql_status.queue_status = PFQL_Q_FILLING;
+
 		if ( !dig_suspend && ctx->pfql_status.do_scan ) {
 			if ( dig_limit )
 				dig_start = time(NULL);
@@ -187,25 +215,34 @@ void* queue_fill_thread(void *arg) {
 					ctx->queue[i].scached = 0;
 					ctx->queue[i].tagged  = 0;
 					b = 1;
+
+					if ( ctx->pfql_status.sort_field )
+						pfql_retr_headers ( ctx, ctx->queue[i].id );
 				}
 			}
 			if ( b )
 				ctx->queue_last_changed = time(NULL);
 			ctx->dig_lastqueue = ctx->pfql_status.cur_queue;
 		}
+		if ( ctx->pfql_status.sort_field!=PFQL_SORT_UNSORTED ) {
+			ctx->pfql_status.queue_status = PFQL_Q_SORTING;
+			pfql_queue_sort ( ctx ); 
+			ctx->pfql_status.queue_status = PFQL_Q_IDLE;
+		}
+
 		sleep ( ctx->pfql_conf.scan_delay );
 	}
-	pthread_mutex_unlock ( &queue_fill_mutex );
+	pthread_mutex_unlock ( &ctx->qfill_mutex );
 	thread_control = TH_STOPPED;
 	pthread_exit(NULL);
 }
 
 // Launches the dig thread
 int queue_fill_start(struct pfql_context_t* ctx) {
-	if ( pthread_mutex_trylock(&queue_fill_mutex)!=0 )
+	if ( pthread_mutex_trylock(&ctx->qfill_mutex)!=0 )
 		return PFQL_ERROR;
 	thread_control = TH_RUNNABLE;
-	pthread_create ( &queue_fill_thread_t, NULL, queue_fill_thread, ctx );
+	pthread_create ( &ctx->qfill_thread, NULL, queue_fill_thread, ctx );
 	return PFQL_OK;
 }
 
@@ -294,10 +331,10 @@ int pfql_num_queues( struct pfql_context_t *ctx ) {
 }
 
 // Changes the current queue
-void pfql_set_queue( struct pfql_context_t *ctx,  int q ) {
+int pfql_set_queue( struct pfql_context_t *ctx, int q ) {
 
-	if ( q > ctx->pfqbe_queue_count() )
-		return;
+	if ( q >= ctx->pfqbe_queue_count() )
+		return PFQL_ERROR;
 	
 	ctx->pfql_status.cur_queue = q;
 	ctx->NUMTAG = 0;
@@ -310,6 +347,7 @@ void pfql_set_queue( struct pfql_context_t *ctx,  int q ) {
 	// Ensure that a scan is made before proceeding, loop 1/5 sec
 	while ( ctx->dig_lastqueue != ctx->pfql_status.cur_queue ) { usleep ( 200000 ); };
 	
+	return PFQL_OK;
 }
 
 // Search for a message; returns -1 if not found
@@ -562,6 +600,8 @@ int pfql_init(struct pfql_context_t *ctx) {
 	ctx->pfql_status.use_envelope    = 0;
 	ctx->pfql_status.use_colors      = 1;
 	ctx->pfql_status.cur_queue       = 0;
+	ctx->pfql_status.sort_field      = PFQL_SORT_UNSORTED;
+	ctx->pfql_status.sort_sense	 = PFQL_SORT_ASC;
 
 	ctx->beptr = 0;
 	ctx->queue = 0;
@@ -588,18 +628,26 @@ int pfql_start(struct pfql_context_t *ctx) {
 
 	/* Alloc resources */
 	ctx->regexp = (regex_t*) malloc (sizeof(regex_t));
-	res = regcomp ( ctx->regexp, "*", 0 );
-	ctx->queue = (struct msg_t*)malloc(sizeof(struct msg_t)*ctx->pfql_conf.msg_max);
-	ctx->beptr = 0;
-
-	if ( !ctx->queue ) {
-		syslog ( LOGLEVEL, "pfqlib: sorry, cannot malloc for %d elements!", ctx->pfql_conf.msg_max );
+	if ( !ctx->regexp ) {
+		syslog ( LOGLEVEL, "pfqlib: sorry, cannot malloc for %d for the regex!", sizeof(regex_t) );
 		return PFQL_MALLOC;
 	}
+	res = regcomp ( ctx->regexp, "*", 0 );
+
+	ctx->queue = (struct msg_t*)malloc(sizeof(struct msg_t)*ctx->pfql_conf.msg_max);
+	if ( !ctx->queue ) {
+		regfree ( ctx->regexp );
+		syslog ( LOGLEVEL, "pfqlib: sorry, cannot malloc for %d elements (queue)!", ctx->pfql_conf.msg_max );
+		return PFQL_MALLOC;
+	}
+
+	ctx->beptr = 0;
+
 	ctx->queue_thread = (struct be_msg_t*)malloc(sizeof(struct be_msg_t)*ctx->pfql_conf.msg_max);
 	if ( !ctx->queue_thread ) {
+		regfree ( ctx->regexp );
 		free ( ctx->queue );
-		syslog ( LOGLEVEL, "pfqlib: sorry, cannot malloc for %d elements!", ctx->pfql_conf.msg_max );
+		syslog ( LOGLEVEL, "pfqlib: sorry, cannot malloc for %d elements (queue_thread)!", ctx->pfql_conf.msg_max );
 		return PFQL_MALLOC;
 	}
 
@@ -664,7 +712,7 @@ int pfql_start(struct pfql_context_t *ctx) {
 
 int pfql_context_destroy(struct pfql_context_t *ctx) {
 
-	pthread_mutex_destroy ( &queue_fill_mutex );
+	pthread_mutex_destroy ( &ctx->qfill_mutex );
 	queue_fill_stop(ctx);
 
 	if (ctx->beptr) {
@@ -682,3 +730,7 @@ int pfql_context_destroy(struct pfql_context_t *ctx) {
 	return PFQL_OK;
 
 }
+
+int pfql_dump ( struct pfql_context_t *ctx, const char *fname ) {
+	return PFQL_OK;
+}
