@@ -9,38 +9,47 @@
 #include <time.h>
 #include <regex.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include "pfregex.h"
+#include <pthread.h>
+
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#define PFQL_VERSION "1.2"
+#define PFQL_VERSION "1.4"
 
+// Structure for configuration
 struct pfql_conf_t {
-	int   max_char;			/* Max char buffer length */
-	short initial_queue;		/* Queue to start with */
-	char  backends_path[200];	/* Path to backends */
-	char  backend_name[200];	/* Backend to use */
-	char  backend_config[200];	/* Backend configuration path */
-	char  backend_progs[200];	/* Backend bin bath */
-	int   msg_max;			/* Maximum number of messages */
-	int   scan_limit;		/* Max secs for a single scan */
-	int   scan_delay;		/* Secs between scans */
-	char  remote_host[200];		/* Remote host (for socket backend only) */
-	int   remote_port;		/* Remote port (for socket backend only) */
+	int   max_char;			/* Max char buffer length 				*/
+	short initial_queue;		/* Queue to start with 					*/
+	char  backends_path[200];	/* Path to backends 					*/
+	char  backend_name[200];	/* Backend to use 					*/
+	char  backend_config[200];	/* Backend configuration path 				*/
+	char  backend_progs[200];	/* Backend bin bath 					*/
+	int   msg_max;			/* Maximum number of messages 				*/
+	int   scan_limit;		/* Max secs for a single scan 				*/
+	int   scan_delay;		/* Secs between scans 					*/
+	char  remote_host[200];		/* Remote host (for socket backend only) 		*/
+	int   remote_port;		/* Remote port (for socket backend only) 		*/
 };
 
+// Status at runtime
 struct pfql_status_t {
-	short auto_wrk_tagged;
-	short wrk_tagged;
-	short ask_confirm;
-	short do_scan;
-	short use_envelope;
-	short use_colors;
-	short cur_queue;
+	short auto_wrk_tagged;		/* Automatically work on tagged messages, if any	*/
+	short wrk_tagged;		/* Work on tagged messages, if any 			*/
+	short ask_confirm;		/* Ask for confirmations 				*/
+	short do_scan;			/* Scan messages 					*/
+	short use_envelope;		/* Use envelope for from/to 				*/
+	short use_colors;		/* Use colors 						*/
+	short cur_queue;		/* Current queue 					*/
+	short sort_field;		/* Sort field 						*/
+	short sort_sense;		/* Sort ascendent/descendent 				*/
+	short queue_status;		/* Tells wether the queue is being digged or not	*/
 };
 
+// pfqlib context private structure
 struct pfql_context_t {
 	struct msg_t *queue;
 	struct be_msg_t *queue_thread;
@@ -53,6 +62,7 @@ struct pfql_context_t {
 	int NUMTAG;
 
 	int thread_control;
+
 	void *beptr;
 	char* (*pfqbe_id)();
 	char* (*pfqbe_version)();
@@ -77,6 +87,9 @@ struct pfql_context_t {
 
 	regex_t  *regexp;
 	int search_mode;
+
+	pthread_t       qfill_thread;
+	pthread_mutex_t qfill_mutex;
 };
 
 /* -----------------
@@ -132,8 +145,11 @@ int		pfql_num_queues(struct pfql_context_t *);
 // Returns queue name
 const char* 	pfql_queue_name(struct pfql_context_t *,int);
 
+// Set queue sorting
+void		pfql_queue_sort(struct pfql_context_t *);
+
 // Sets current queue
-void		pfql_set_queue(struct pfql_context_t *,int);
+int		pfql_set_queue(struct pfql_context_t *,int);
 
 // The last time the queue changed
 time_t		pfql_queue_last_changed(struct pfql_context_t *);
@@ -192,9 +208,17 @@ void		pfql_msg_searchandtag(struct pfql_context_t *,const char*);
 // Returns message at a given position in the queue
 struct msg_t*	pfql_msg_at(struct pfql_context_t *,int);
 
-// Returns message position
+// Returns a message given its ID
 struct msg_t*	pfql_msg(struct pfql_context_t *,const char*);
 
+// Store messages in a file
+int		pfql_dump(struct pfql_context_t *, const char* );
+
+// Queue status
+#define		PFQL_Q_FILLING		0
+#define		PFQL_Q_IDLE		1
+#define		PFQL_Q_SORTING		2
+
 // Return codes
 #define		PFQL_OK			0
 #define		PFQL_ERROR		-1
@@ -206,6 +230,16 @@ struct msg_t*	pfql_msg(struct pfql_context_t *,const char*);
 #define		PFQL_NOBE		-6
 #define		PFQL_BEINIT		-7
 #define		PFQL_MSGNOTEX		-1
+#define		PFQL_NOFILE		-8
+
+// Sort fields
+#define		PFQL_SORT_UNSORTED	0
+#define		PFQL_SORT_FROM		1
+#define 	PFQL_SORT_TO		2
+#define 	PFQL_SORT_SUBJECT	3
+
+#define		PFQL_SORT_ASC		1
+#define		PFQL_SORT_DESC		-1
 
 #ifdef __cplusplus
 }
