@@ -11,11 +11,15 @@
 #define	CMD_QNAME	"QNAME"
 #define CMD_SETQ	"SETQ"
 #define CMD_LSTIDS	"LSTIDS"
+#define CMD_PATH	"MSGPATH"
 #define CMD_FROM	"MSGFROM"
 #define CMD_TO		"MSGTO"
 #define CMD_SUBJ	"MSGSUBJ"
 #define CMD_MSGID	"MSGID"
+#define CMD_STATUS	"MSGSTAT"
 #define CMD_QUIT	"QUIT"
+#define CMD_BODY	"BODY"
+#define CMD_LASTCHG	"LASTC"
 
 #define CMD_MSGDEL	"MSGDEL"
 #define	CMD_MSGREQ	"MSGREQ"
