@@ -2,12 +2,6 @@
 #ifndef __PFREGEX_H
 #define __PFREGEX_H
 
-#include <sys/types.h>
-#include <regex.h>
-
-regex_t	*regexp;
-int     search_mode;
-
 #define REGEXP_LEN	200
 
 #define SM_FROM		1
